import type { IRenderer } from './AbstractRenderer';
import { Camera } from './camera';
import { DisplayObject } from './display-objects';
import type { IChildNode } from './dom';
import { Document, EventTarget } from './dom';
import { CustomElementRegistry } from './dom/CustomElementRegistry';
import type { ICanvas } from './dom/interfaces';
import { ContextService, EventService, RenderingContext, RenderingService } from './services';
import type { PointLike } from './shapes';
import type { Cursor, InteractivePointerEvent } from './types';
import { CanvasConfig } from './types';
export declare enum CanvasEvent {
    READY = "ready",
    BEFORE_RENDER = "beforerender",
    AFTER_RENDER = "afterrender",
    BEFORE_DESTROY = "beforedestroy",
    AFTER_DESTROY = "afterdestroy",
    RESIZE = "resize",
    DIRTY_RECTANGLE = "dirtyrectangle"
}
/**
 * can be treated like Window in DOM
 * provide some extra methods like `window`, such as:
 * * `window.requestAnimationFrame`
 * * `window.devicePixelRatio`
 *
 * prototype chains: Canvas(Window) -> EventTarget
 */
export declare class Canvas extends EventTarget implements ICanvas {
    /**
     * child container of current canvas, use hierarchy container
     */
    container: import("mana-syringe").Syringe.Container;
    /**
     * window.document
     */
    document: Document;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry
     */
    customElements: CustomElementRegistry;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame
     */
    requestAnimationFrame: (callback: FrameRequestCallback) => number;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/cancelAnimationFrame
     */
    cancelAnimationFrame: (handle: number) => void;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */
    devicePixelRatio: number;
    /**
     * whether the runtime supports PointerEvent?
     * if not, the event system won't trigger pointer events like `pointerdown`
     */
    supportsPointerEvents: boolean;
    /**
     * whether the runtime supports TouchEvent?
     * if not, the event system won't trigger touch events like `touchstart`
     */
    supportsTouchEvents: boolean;
    /**
     * is this native event a TouchEvent?
     */
    isTouchEvent: (event: InteractivePointerEvent) => event is TouchEvent;
    /**
     * is this native event a MouseEvent?
     */
    isMouseEvent: (event: InteractivePointerEvent) => event is MouseEvent;
    /**
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Element
     */
    Element: typeof DisplayObject;
    /**
     * rAF in auto rendering
     */
    private frameId?;
    /**
     * cache here since inversify's resolving is very slow
     */
    private eventService;
    private renderingService;
    private inited;
    private readyPromise;
    private resolveReadyPromise;
    constructor(config: CanvasConfig);
    private initRenderingContext;
    private initDefaultCamera;
    getConfig(): Partial<CanvasConfig>;
    getContainer(): import("mana-syringe").Syringe.Container;
    /**
     * get the root displayObject in scenegraph
     * @alias this.document.documentElement
     */
    getRoot(): import("./display-objects").Group;
    /**
     * get the camera of canvas
     */
    getCamera(): Camera;
    getContextService(): ContextService<unknown>;
    getEventService(): EventService;
    getRenderingService(): RenderingService;
    getRenderingContext(): RenderingContext;
    getStats(): {
        total: number;
        rendered: number;
    };
    get ready(): Promise<any>;
    destroy(destroyScenegraph?: boolean, skipTriggerEvent?: boolean): void;
    /**
     * compatible with G 3.0
     * @deprecated
     * @alias resize
     */
    changeSize(width: number, height: number): void;
    resize(width: number, height: number): void;
    appendChild<T extends IChildNode>(child: T, index?: number): T;
    insertBefore<T extends IChildNode, N extends IChildNode>(newChild: T, refChild: N | null): T;
    removeChild<T extends IChildNode>(child: T, destroy?: boolean): T;
    removeChildren(destroy?: boolean): void;
    render(): void;
    private run;
    private initRenderer;
    private loadCommonContainerModule;
    private loadRendererContainerModule;
    setRenderer(renderer: IRenderer): Promise<void>;
    setCursor(cursor: Cursor): void;
    private unmountChildren;
    private mountChildren;
    client2Viewport(client: PointLike): PointLike;
    viewport2Client(canvas: PointLike): PointLike;
    viewport2Canvas(viewport: PointLike): PointLike;
    canvas2Viewport(canvas: PointLike): PointLike;
    /**
     * @deprecated
     * @alias client2Viewport
     */
    getPointByClient(clientX: number, clientY: number): PointLike;
    /**
     * @deprecated
     * @alias viewport2Client
     */
    getClientByPoint(x: number, y: number): PointLike;
}
//# sourceMappingURL=Canvas.d.ts.map